package de.mdelab.expressiondialogs;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.emf.edit.command.CommandParameter;
import org.eclipse.emf.edit.command.RemoveCommand;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.edit.provider.ItemPropertyDescriptor;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryContentProvider;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TreeItem;

import de.mdelab.mlcallactions.ActivityCallAction;
import de.mdelab.mlcallactions.MlcallactionsFactory;
import de.mdelab.mlcallactions.MlcallactionsPackage;
import de.mdelab.mlexpressions.MLExpression;
import de.mdelab.mlsdm.Activity;
import de.mdelab.mlsdm.ActivityEdge;
import de.mdelab.mlsdm.ActivityEdgeGuardEnum;
import de.mdelab.mlsdm.ActivityParameter;
import de.mdelab.mlsdm.ActivityParameterDirectionEnum;
import de.mdelab.mlsdm.MlsdmPackage;
import de.mdelab.mlsdm.diagram.part.MlsdmDiagramEditorPlugin;
import de.mdelab.mlstorypatterns.MlstorypatternsUtils;
import de.mdelab.mlstorypatterns.StoryPatternObject;

public class MlsdmEditExpressionDialog extends CommonEditExpressionDialog {

	private Activity	activity;

	public MlsdmEditExpressionDialog(Shell parent) {
		super(parent);
		/*
		 * Add a default source viewer for all expression languages that do not
		 * have one.
		 */
		for (String expressionLanguage : MlstorypatternsUtils.getAvailableExpressionLanguages())
		{
			if (!this.sourceViewerProviders.containsKey(expressionLanguage))
			{
				this.sourceViewerProviders.put(expressionLanguage, new SourceViewerProvider());
			}
		}

	}
	
	abstract class ExpressionView extends AbstractView
	{
		@Override
		public Command getExtraCommand4Addition(TransactionalEditingDomain domain)
		{
			if (MlsdmEditExpressionDialog.this.currentTreeValue instanceof ActivityEdge
					&& MlsdmEditExpressionDialog.this.feature.equals(MlsdmPackage.eINSTANCE.getActivityEdge_GuardExpression())
					&& ((ActivityEdge) MlsdmEditExpressionDialog.this.currentTreeValue).getGuardType() != ActivityEdgeGuardEnum.BOOLEAN)
			{
				return domain.createCommand(SetCommand.class, new CommandParameter(MlsdmEditExpressionDialog.this.currentTreeValue,
						MlsdmPackage.eINSTANCE.getActivityEdge_GuardType(), ActivityEdgeGuardEnum.BOOLEAN));
			}

			return null;
		}

		@Override
		public Command getExtraCommand4Removal(TransactionalEditingDomain domain)
		{
			EObject container = MlsdmEditExpressionDialog.this.currentTreeValue.eContainer();
			if (MlsdmEditExpressionDialog.this.currentTreeValue instanceof MLExpression && container instanceof ActivityEdge
					&& MlsdmEditExpressionDialog.this.feature.equals(MlsdmPackage.eINSTANCE.getActivityEdge_GuardExpression())
					&& ((ActivityEdge) container).getGuardType() == ActivityEdgeGuardEnum.BOOLEAN)
			{
				return domain.createCommand(SetCommand.class,
						new CommandParameter(container, MlsdmPackage.eINSTANCE.getActivityEdge_GuardType(),
								ActivityEdgeGuardEnum.NONE));
			}

			return null;
		}
	}

	
	class ActivityCallActionView extends CallActionView
	{
		@Override
		public void adaptInputElements()
		{
			super.adaptInputElements();
			
			if (((ActivityCallAction) MlsdmEditExpressionDialog.this.currentTreeValue).getReturnValueParameter() != null)
			{
				int i;
				
				for (i = 0; i < MlsdmEditExpressionDialog.this.activityParameterValues.toArray().length; i++)
				{
					if (((ActivityParameter) MlsdmEditExpressionDialog.this.activityParameterValues.toArray()[i]).getName().equals(
							((ActivityCallAction) MlsdmEditExpressionDialog.this.currentTreeValue).getReturnValueParameter().getName()))
					{
						break;
					}
				}
				MlsdmEditExpressionDialog.this.addOrResetActivityParameterFieldWithCheckBox(i);
			}
			else
			{
				MlsdmEditExpressionDialog.this.addOrResetActivityParameterFieldWithCheckBox(-1);
			}
			
			MlsdmEditExpressionDialog.this.activityCheckBox.setSelection(((ActivityCallAction) MlsdmEditExpressionDialog.this.currentTreeValue).isImportOutputParameters());

			MlsdmEditExpressionDialog.this.activityParameterCombo.addSelectionListener(MlsdmEditExpressionDialog.this
					.createComboSelectionListener(MlcallactionsPackage.eINSTANCE.getActivityCallAction_ReturnValueParameter(),
							MlsdmEditExpressionDialog.this.activityParameterCombo, MlsdmEditExpressionDialog.this.activityParameterValues));
			MlsdmEditExpressionDialog.this.activityCheckBox.addSelectionListener(MlsdmEditExpressionDialog.this
					.createButtonSelectionListener(MlcallactionsPackage.eINSTANCE.getActivityCallAction_ReturnValueParameter(),
							MlsdmEditExpressionDialog.this.activityCheckBox, ((ActivityCallAction) MlsdmEditExpressionDialog.this.currentTreeValue)));
		}

		@Override
		public EObject create() {
			return MlcallactionsFactory.eINSTANCE.createActivityCallAction();
		}
	}
	
	@Override
	protected void addOrResetActivityfield(int selection)
	{
		this.activityField = new Composite(this.lower, SWT.NONE);
		this.activityField.setLayout(new GridLayout(1, true));
		this.activityField.setLayoutData(new GridData(GridData.FILL, GridData.BEGINNING, true, false));

		this.activityLabel = new Label(this.activityField, SWT.NONE);
		this.activityLabel.setText("Activity:");
		this.activityLabel.setLayoutData(new GridData(GridData.BEGINNING, GridData.BEGINNING, true, false));

		this.activityCombo = new Combo(this.activityField, SWT.READ_ONLY);
		GridData gdTxtvarname = new GridData(GridData.FILL, GridData.FILL, true, true);
		gdTxtvarname.verticalSpan = 4;
		this.activityCombo.setLayoutData(gdTxtvarname);

		if (this.activityValues!=null){
			if (this.activityValues.size()>0){
				for (EObject a : this.activityValues)
				{
					if (((Activity) a).getName() != null)
					{
						this.activityCombo.add(((Activity) a).getName());
					}
				}
		
				this.activityCombo.select(selection);
			}
		}
	}
	
	@Override
	protected void addOrResetActivityParameterField(int selection)
	{
		this.activityParameterField = new Composite(this.lower, SWT.NONE);
		this.activityParameterField.setLayout(new GridLayout(1, true));
		this.activityParameterField.setLayoutData(new GridData(GridData.FILL, GridData.BEGINNING, true, false));

		this.activityParameterLabel = new Label(this.activityParameterField, SWT.NONE);
		this.activityParameterLabel.setText("Return Parameter:");
		this.activityParameterLabel.setLayoutData(new GridData(GridData.BEGINNING, GridData.BEGINNING, true, false));

		this.activityParameterCombo = new Combo(this.activityParameterField, SWT.READ_ONLY);
		GridData gdTxtvarname = new GridData(GridData.FILL, GridData.FILL, true, true);
		gdTxtvarname.verticalSpan = 4;
		this.activityParameterCombo.setLayoutData(gdTxtvarname);

		for (EObject a : this.activityParameterValues)
		{
			if (((ActivityParameter) a).getName() != null)
			{
				this.activityParameterCombo.add(((ActivityParameter) a).getName());
			}
		}

		this.activityParameterCombo.select(selection);
	}
	
	@Override
	protected void addOrResetActivityParameterFieldWithCheckBox(int selection)
	{
		this.activityParameterField = new Composite(this.lower, SWT.NONE);
		this.activityParameterField.setLayout(new GridLayout(1, true));
		this.activityParameterField.setLayoutData(new GridData(GridData.FILL, GridData.BEGINNING, true, false));

		this.activityParameterLabel = new Label(this.activityParameterField, SWT.NONE);
		this.activityParameterLabel.setText("Return Parameter:");
		this.activityParameterLabel.setLayoutData(new GridData(GridData.BEGINNING, GridData.BEGINNING, true, false));

		
		this.activityParameterCombo = new Combo(this.activityParameterField, SWT.READ_ONLY);
		GridData gdTxtvarname = new GridData(GridData.FILL, GridData.FILL, true, true);
		gdTxtvarname.verticalSpan = 4;
		this.activityParameterCombo.setLayoutData(gdTxtvarname);

		for (EObject a : this.activityParameterValues)
		{
			if (((ActivityParameter) a).getName() != null)
			{
				this.activityParameterCombo.add(((ActivityParameter) a).getName());
			}
		}

		this.activityParameterCombo.select(selection);
		
		this.activityCheckBox = new Button(this.activityParameterField, SWT.CHECK);
		this.activityCheckBox.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, true));
		this.activityCheckBox.setText("import output parameters");
	}
	
	@Override
	protected void changeTreePositions(boolean direction){ //direction = true = "up"; direction = false = "down"
		if (MlsdmEditExpressionDialog.this.callActionTypeCombobox.getSelectionIndex() == -1)
		{
			return;
		}
		TreeItem ti = currentTreeItem;
		StoryPatternObject spo = (StoryPatternObject) ti.getParentItem().getData();
		TransactionalEditingDomain domain;
		int size = spo.getConstraints().size();
		int oldPos = -1;
		int newPos = -1;
		MLExpression[] expressions = new MLExpression[size];
		
		for (int i=0; i<size;i++){
			expressions[i] = spo.getConstraints().get(i);
			if (expressions[i].equals(ti.getData())){
				if (direction){
					newPos = (i-1+size)%size;
				}
				else
				{
					newPos = (i+1)%size;
				}
				oldPos = i;
			}
		}
		// remove old constraints
		for (int j = 0; j<size;j++){
			domain = TransactionUtil.getEditingDomain(spo);
			domain.getCommandStack().execute(
					new RemoveCommand(domain, spo, spo.eClass()
							.getEStructuralFeature("constraints"), expressions[j]));
		}
		
		// change positions
		MLExpression ex = expressions[oldPos];
		expressions[oldPos] = expressions[newPos];
		expressions[newPos]=ex;
		
		// add new constraints
		for (int j = 0; j<size;j++){
			domain = TransactionUtil.getEditingDomain(spo);
			domain.getCommandStack().execute(
					new AddCommand(domain, spo, spo.eClass()
							.getEStructuralFeature("constraints"), expressions[j]));
		}
		
		if (callActionHierarchie.getTree().getItem(0).getItemCount() > 0)
		{
			for (TreeItem treeItem : callActionHierarchie.getTree().getItem(0).getItems())
			{
				if (treeItem.getData().equals(expressions[oldPos]))
				{
					callActionHierarchie.getTree().select(treeItem);
					selectionChanged(treeItem);
					break;
				}
			}
		}
	}
	@Override
	protected void addOrResetStringExpressionEditor(String string, String content)
	{
		this.constraintLanguages = MlstorypatternsUtils.getAvailableExpressionLanguages();
		super.addOrResetStringExpressionEditor(string, content);
	}
	
	@Override
	protected void createCallActionHierarchie(){
		this.callActionHierarchie = new TreeViewer(this.upper, SWT.NONE); 

		GridData gridData = new GridData(GridData.FILL, GridData.FILL, true, true, 3, 1);
		gridData.minimumWidth = 500;
		gridData.minimumHeight = 200;
		gridData.widthHint = 500;
		gridData.heightHint = 200;
		this.callActionHierarchie.getTree().setLayoutData(gridData);
		this.callActionHierarchie.setContentProvider(new AdapterFactoryContentProvider(MlsdmDiagramEditorPlugin.getInstance()
				.getItemProvidersAdapterFactory()));
		this.callActionHierarchie.setLabelProvider(new AdapterFactoryLabelProvider.ColorProvider(MlsdmDiagramEditorPlugin
				.getInstance().getItemProvidersAdapterFactory(), this.callActionHierarchie));
		//TODO:??
		//JOptionPane.showInputDialog("MlsdmEditExpr ##### " + this.expressionOwner);
		if (this.expressionOwner!=null)
		{
			this.callActionHierarchie.setInput(this.expressionOwner);
			ViewerFilter[] viewerFilters = new ViewerFilter[1];
			viewerFilters[0] = new ViewerFilter()
			{

				@Override
				public boolean select(Viewer viewer, Object parentElement, Object element)
				{
					//if (element instanceof MLStringExpression && element != MlsdmEditExpressionDialog.this.expression)
					//	return false;
					
					EObject container = (EObject) element;
					while (container!=null && container != MlsdmEditExpressionDialog.this.expression)
						container = container.eContainer();
					if (container == MlsdmEditExpressionDialog.this.expression)
						return true;
					return false;
				}
			};
			this.callActionHierarchie.setFilters(viewerFilters);
		}

		// creating Listener for the TreeItems
		SelectionListener stdTreeItemlistener = createSelectionListener();
		this.callActionHierarchie.getTree().addSelectionListener(stdTreeItemlistener);

	}
	
	@Override
	protected SelectionListener createSelectionListener(){
		SelectionListener stdTreeItemlistener = new SelectionListener()
		{
			@Override
			public void widgetDefaultSelected(SelectionEvent event)
			{
				this.widgetSelected(event);
			}

			@Override
			public void widgetSelected(SelectionEvent event)
			{
				MlsdmEditExpressionDialog.this.processCurrentInput();
				MlsdmEditExpressionDialog.this.selectionChanged(event.item);
				upButton.setEnabled(false);
				downButton.setEnabled(false);		
				
				if (((TreeItem)event.item).getParentItem()!= null)
					if (((TreeItem)event.item).getParentItem().getData() instanceof StoryPatternObject){
						upButton.setEnabled(true);
						downButton.setEnabled(true);
					}
			}
		};
		return stdTreeItemlistener;
	}
	
	@Override
	protected void createValues(){

		if (this.rootContainer!=null)
			this.activityValues = ItemPropertyDescriptor.getReachableObjectsOfType(this.rootContainer,
					MlsdmPackage.eINSTANCE.getActivity());
		
		if (this.rootContainer!=null)
			this.activityParameterValues = ItemPropertyDescriptor.getReachableObjectsOfType(this.rootContainer,
					MlsdmPackage.eINSTANCE.getActivityParameter());

	}

	public void setActivity(Activity activity)
	{
		this.activity = activity;
		this.setRootContainer(activity);
	}

	@Override
	protected Map<String, EClassifier> getContextInformation()
	{
		Map<String, EClassifier> contextInfos = new HashMap<String, EClassifier>();

		if (this.activity != null)
		{
			/*
			 * THIS and parameters
			 */
			if (this.activity/*.getSpecification()*/ != null)
			{
				EClass thisClass = null;//this.activity.getSpecification().getEContainingClass();

				if (thisClass != null)
				{
					contextInfos.put("this", thisClass);
				}

				for (ActivityParameter parameter : this.activity.getParameters())
				{
					if (!parameter.getDirection().getName().equals("IN") && parameter.getName() != null && !"".equals(parameter) && parameter.getType() != null)
					{
						contextInfos.put(parameter.getName(), parameter.getType());
					}
				}
			}
			else
			{
				for (ActivityParameter parameter : this.activity.getParameters())
				{
					if ((parameter.getDirection() == ActivityParameterDirectionEnum.INOUT || parameter.getDirection() == ActivityParameterDirectionEnum.OUT)
							&& parameter.getName() != null && !"".equals(parameter) && parameter.getType() != null)
					{
						contextInfos.put(parameter.getName(), parameter.getType());
					}
				}
			}

			/*
			 * Variables created in story patterns and
			 * VariableDeclarationActions
			 */
			TreeIterator<EObject> it = this.activity.eAllContents();

			while (it.hasNext())
			{
				EObject eObject = it.next();

				if (eObject instanceof StoryPatternObject)
				{
					StoryPatternObject spo = (StoryPatternObject) eObject;

					if (spo.getName() != null && !"".equals(spo.getName()) && spo.getType() != null)
					{
						contextInfos.put(spo.getName(), spo.getType());
					}
				}
			}
		}

		return contextInfos;
	}
	@Override
	protected Control createDialogArea(Composite parent)
	{
		return super.createDialogArea(parent);
	}
}
