package de.mdelab.expressiondialogs;


import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EOperation;
import org.eclipse.emf.ecore.EParameter;
import org.eclipse.emf.ecoretools.diagram.edit.parts.EReferenceUtils;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gmf.runtime.diagram.ui.commands.ICommandProxy;
import org.eclipse.gmf.runtime.diagram.ui.editparts.ConnectionNodeEditPart;
import org.eclipse.gmf.runtime.diagram.ui.editparts.IGraphicalEditPart;
import org.eclipse.gmf.runtime.draw2d.ui.figures.PolylineConnectionEx;
import org.eclipse.gmf.runtime.notation.View;
import org.eclipse.swt.SWT;

import de.mdelab.mlcore.MLNamedElement;
import de.mdelab.mlsdm.Activity;
import de.mdelab.mlsdm.ActivityEdgeGuardEnum;
import de.mdelab.mlstorypatterns.MatchTypeEnum;
import de.mdelab.mlstorypatterns.ModifierEnum;
import de.mdelab.mlstorypatterns.StoryPatternElement;
import de.mdelab.mlstorypatterns.diagram.custom.ILineIsHideableFigure;
import de.mdelab.mlstorypatterns.diagram.custom.MlstorypatternsDiagramConstants;
import de.mdelab.mlstorypatterns.diagram.custom.edit.commands.UpdateMergedLinkDeferredCommand;
import de.mdelab.mlstorypatterns.diagram.figures.CrossFigure;

public class Utility
{

	public static String GuardEnum2String(ActivityEdgeGuardEnum guardEnum)
	{
		return guardEnum.toString().toLowerCase().replaceAll("_", " ");
	}

	public static ActivityEdgeGuardEnum String2GuardEnum(String text)
	{
		return ActivityEdgeGuardEnum.valueOf(text.replaceAll(" ", "_").toUpperCase());
	}

	public static String adaptStoryPatternElementModifierText(IGraphicalEditPart editPart)
	{
		StoryPatternElement spo = (StoryPatternElement) ((View) editPart.getModel()).getElement();

		switch (spo.getModifier())
		{
			case CREATE:
			{
				return "<<create>>";
			}
			case DESTROY:
			{
				return "<<destroy>>";
			}
			default:
			{
				return "";
			}
		}

	}

	public static void adaptLineStyle(PolylineConnectionEx figure, MatchTypeEnum spMatch)
	{
		for (Object o : figure.getChildren())
		{
			if (o instanceof CrossFigure)
			{
				figure.getChildren().remove(o);
				break;
			}
		}

		switch (spMatch)
		{
		/*	case NONE:
			{
				figure.setLineStyle(SWT.LINE_SOLID);
				break;
			}*/
			case OPTIONAL:
			{
				figure.setLineStyle(SWT.LINE_CUSTOM);
				figure.setLineDash(new int[]
				{
						15, 10
				});
				break;
			}
	/*		case NEGATIVE:
			{
				CrossFigure crossFigure = new CrossFigure();

				Dimension size = new Dimension(20, 20);
				crossFigure.setMinimumSize(size);
				crossFigure.setMaximumSize(size);
				crossFigure.setPreferredSize(size);

				figure.add(crossFigure, new ConnectionLocator(figure, ConnectionLocator.MIDDLE));
			}*/
			default:
			{
				figure.setLineStyle(SWT.LINE_SOLID);
				break;
			}
		}

	}

	public static void adaptColor(IFigure figure, ModifierEnum spMod)
	{
		// StoryPatternModifierEnumeration spMod = ((StoryPatternElement)
		// ((View) graphEditPart.getModel()).getElement()).getModifier();

		switch (spMod)
		{
			case CREATE:
			{
				figure.setForegroundColor(MlstorypatternsDiagramConstants.MODIFIER_CREATE_COLOR);
				break;
			}
			case DESTROY:
			{
				figure.setForegroundColor(MlstorypatternsDiagramConstants.MODIFIER_DESTROY_COLOR);
				break;
			}
			case NONE:
			{
				figure.setForegroundColor(MlstorypatternsDiagramConstants.MODIFIER_NONE_COLOR);
				break;
			}
			default:
			{
				figure.setForegroundColor(MlstorypatternsDiagramConstants.MODIFIER_NONE_COLOR);
				break;
			}
		}
	}

	/**
	 * Zum größten Teil kopiert aus
	 * org.eclipse.emf.ecoretools.diagram.edit.parts
	 * .EReferenceUtils.trackReferences
	 */
	public static void trackMergedLinks(ConnectionNodeEditPart linkEditPart, EObject opLink, boolean trackFully)
	{
		EditPart targetPart = linkEditPart.getTarget();
		if (!(targetPart instanceof IGraphicalEditPart))
		{
			return;
		}
		ConnectionNodeEditPart oppositeEditPart = null;
		for (Object part : ((IGraphicalEditPart) targetPart).getSourceConnections())
		{
			if (!(part instanceof ConnectionNodeEditPart))
			{
				continue;
			}
			if (((ConnectionNodeEditPart) part).resolveSemanticElement() == opLink) // ((StoryPatternLink)
																					// semanticElement).getOppositeStoryPatternLink()
			{
				oppositeEditPart = (ConnectionNodeEditPart) part;
				break;
			}
		}
		if (oppositeEditPart == null)
		{
			return;
		}
		// Hide line of the non selected reference
		ConnectionNodeEditPart masterPart;
		ConnectionNodeEditPart slavePart;
		if (oppositeEditPart.getSelected() != EditPart.SELECTED_NONE)
		{
			((ILineIsHideableFigure) linkEditPart.getFigure()).setHideLine(true);
			((ILineIsHideableFigure) oppositeEditPart.getFigure()).setHideLine(false);
			masterPart = oppositeEditPart;
			slavePart = linkEditPart;
		}
		else
		{
			((ILineIsHideableFigure) oppositeEditPart.getFigure()).setHideLine(true);
			((ILineIsHideableFigure) linkEditPart.getFigure()).setHideLine(false);
			masterPart = linkEditPart;
			slavePart = oppositeEditPart;
		}

		if (trackFully)
		{
			TransactionalEditingDomain editingDomain = masterPart.getEditingDomain();
			Command cmd = new ICommandProxy(new UpdateMergedLinkDeferredCommand(editingDomain, masterPart, slavePart));
			EReferenceUtils.executeCommand(cmd, masterPart);
		}
	}

	public static String getSpecificationString(EOperation specification)
	{
		String text = "";
		if (specification != null && specification.getEContainingClass() != null
				&& specification.getEContainingClass().getEPackage() != null)
		{
			/*
			 * Operation name
			 */
			text = specification.getEContainingClass().getName() + "." + specification.getName();

			/*
			 * Parameters
			 */
			text += "(";

			if (!specification.getEParameters().isEmpty())
			{
				for (EParameter parameter : specification.getEParameters())
				{
					text += parameter.getEType().getName() + " " + parameter.getName() + ", ";
				}

				text = text.substring(0, text.length() - 2);
			}
			text += ")";

			/*
			 * Return value
			 */
			if (specification.getEType() != null)
			{
				text += ":" + specification.getEType().getName();
			}
		}
		else
		{
			text = "";
		}

		return text;
	}

	public static String getFilterId(MLNamedElement nElem)
	{
		return nElem.eClass().getName() + ": ";
	}

	/**
	 * Returns the activity that contains this element.
	 * 
	 * @param storyPatternLink
	 * @return
	 */
	public static Activity getActivity(MLNamedElement element)
	{
		do
		{
			element = (MLNamedElement) element.eContainer();
		}
		while (element != null && !(element instanceof Activity));

		return (Activity) element;
	}
}
