package de.mdelab.expressiondialogs.actions;



import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.common.command.UnexecutableCommand;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.emf.edit.command.CommandParameter;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;
import org.eclipse.gmf.runtime.common.core.command.CommandResult;
import org.eclipse.gmf.runtime.emf.commands.core.command.AbstractTransactionalCommand;
import org.eclipse.gmf.runtime.notation.View;
import org.eclipse.jface.window.Window;

import de.mdelab.expressiondialogs.MlsdmEditExpressionDialog;
import de.mdelab.mlexpressions.MLExpression;
import de.mdelab.mlexpressions.MLStringExpression;
import de.mdelab.mlexpressions.MlexpressionsFactory;
import de.mdelab.mlsdm.Activity;
import de.mdelab.mlsdm.MlsdmPackage;
import de.mdelab.mlsdm.diagram.edit.parts.ExpressionLinkEditPart;
import de.mdelab.mlstorypatterns.ExpressionLink;
import de.mdelab.mlstorypatterns.MatchTypeEnum;
import de.mdelab.mlstorypatterns.MlstorypatternsPackage;

public class EditStoryPatternExpressionLinkAction extends EditExpressionAction
{

	@Override
	protected MLExpression getExpression()
	{
		MLExpression exp = (MLExpression) (((ExpressionLink) this.getExpressionOwner()).getExpression());
		//If there is no expression, create one
		if (exp == null){
			MLStringExpression exp2 = null;
			exp2 = MlexpressionsFactory.eINSTANCE.createMLStringExpression();
			exp2.setExpressionLanguageID("OCL");
			CompoundCommand cc = new CompoundCommand();
			ExpressionLink ae = (ExpressionLink) this.getExpressionOwner();
			TransactionalEditingDomain domain = TransactionUtil.getEditingDomain(ae);
			cc.append(domain.createCommand(SetCommand.class,
						new CommandParameter(ae, MlstorypatternsPackage.eINSTANCE.getExpressionLink_Expression(),
								exp2)));
			Command command = cc.unwrap();
			if (!command.equals(UnexecutableCommand.INSTANCE))
			{
				domain.getCommandStack().execute(command);
			}
			exp = exp2;
		}
		return exp;
	}

	@Override
	protected TransactionalEditingDomain getEditingDomain()
	{
		return ((ExpressionLinkEditPart) getStructuredSelection().getFirstElement()).getEditingDomain();
	}

	@Override
	protected void setNewExpression(MLExpression newExpression)
	{
		ExpressionLink edge = (ExpressionLink) this.getExpressionOwner();

		edge.setExpression(newExpression);
	}

	@Override
	protected EObject getExpressionOwner()
	{
		//JOptionPane.showInputDialog(((ExpressionLink) ((View) ((ExpressionLinkEditPart) getStructuredSelection().getFirstElement()).getModel()).getElement()));
		return ((View) ((ExpressionLinkEditPart) getStructuredSelection().getFirstElement()).getModel()).getElement();
	}

	@Override
	protected void prepareDialog()
	{
		super.prepareDialog();
		editExpressionDialog.setStructuralFeature(MlstorypatternsPackage.eINSTANCE.getExpressionLink_Expression());
	}

	@Override
	protected EClassifier getExpectedClassifier()
	{
		return EcorePackage.eINSTANCE.getEBoolean();
	}

	@Override
	protected EClassifier getContextClassifier()
	{
		// No context classifier can be specified.
		return null;
	}
	@Override
	protected void doRun(IProgressMonitor progressMonitor){
	final ExpressionLink expressionLink = (ExpressionLink) this.getExpressionOwner();
	AbstractTransactionalCommand command = new AbstractTransactionalCommand(getEditingDomain(), "Set GuardExpression",
			null)
	{
		protected CommandResult doExecuteWithResult(IProgressMonitor monitor, IAdaptable info) throws ExecutionException
		{
			MlsdmEditExpressionDialog editExpressionDialog = new MlsdmEditExpressionDialog(null);
			editExpressionDialog.setExpectedClassifier(EcorePackage.eINSTANCE.getEBoolean());
			editExpressionDialog.setExpressionOwner(expressionLink);
			if (expressionLink.getExpression()==null)
			{
				MLStringExpression exp2 = null;
				exp2 = MlexpressionsFactory.eINSTANCE.createMLStringExpression();
				exp2.setExpressionLanguageID("OCL");
				expressionLink.setExpression(exp2);
				editExpressionDialog.setExpression(exp2);
			}
			else
			{
				editExpressionDialog.setExpression(expressionLink.getExpression());
			}
			editExpressionDialog.setStructuralFeature(MlsdmPackage.eINSTANCE.getActivityEdge_GuardExpression());

			if (editExpressionDialog.open() == Window.OK)
			{			

				expressionLink.setExpression(editExpressionDialog.getExpression());
			}
			else
			{
				expressionLink.setMatchType(MatchTypeEnum.DEFAULT);
			}

			return CommandResult.newOKCommandResult();
		}
	};
	execute(command);
	((ExpressionLinkEditPart) getStructuredSelection().getFirstElement()).addNotify();
}
}
